/* Generated by dbus-binding-tool; do not edit! */

#include <glib.h>
#include <dbus/dbus-glib.h>

G_BEGIN_DECLS

#ifndef _DBUS_GLIB_ASYNC_DATA_FREE
#define _DBUS_GLIB_ASYNC_DATA_FREE
static
#ifdef G_HAVE_INLINE
inline
#endif
void
_dbus_glib_async_data_free (gpointer stuff)
{
	g_slice_free (DBusGAsyncData, stuff);
}
#endif

#ifndef DBUS_GLIB_CLIENT_WRAPPERS_org_ayatana_indicator_me_service
#define DBUS_GLIB_CLIENT_WRAPPERS_org_ayatana_indicator_me_service

static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
org_ayatana_indicator_me_service_watch (DBusGProxy *proxy, GError **error)

{
  return dbus_g_proxy_call (proxy, "Watch", error, G_TYPE_INVALID, G_TYPE_INVALID);
}

typedef void (*org_ayatana_indicator_me_service_watch_reply) (DBusGProxy *proxy, GError *error, gpointer userdata);

static void
org_ayatana_indicator_me_service_watch_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
  (*(org_ayatana_indicator_me_service_watch_reply)data->cb) (proxy, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
org_ayatana_indicator_me_service_watch_async (DBusGProxy *proxy, org_ayatana_indicator_me_service_watch_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "Watch", org_ayatana_indicator_me_service_watch_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_INVALID);
}
static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
org_ayatana_indicator_me_service_status_icons (DBusGProxy *proxy, char ** OUT_icons, GError **error)

{
  return dbus_g_proxy_call (proxy, "StatusIcons", error, G_TYPE_INVALID, G_TYPE_STRING, OUT_icons, G_TYPE_INVALID);
}

typedef void (*org_ayatana_indicator_me_service_status_icons_reply) (DBusGProxy *proxy, char * OUT_icons, GError *error, gpointer userdata);

static void
org_ayatana_indicator_me_service_status_icons_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  char * OUT_icons;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_STRING, &OUT_icons, G_TYPE_INVALID);
  (*(org_ayatana_indicator_me_service_status_icons_reply)data->cb) (proxy, OUT_icons, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
org_ayatana_indicator_me_service_status_icons_async (DBusGProxy *proxy, org_ayatana_indicator_me_service_status_icons_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "StatusIcons", org_ayatana_indicator_me_service_status_icons_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_INVALID);
}
static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
org_ayatana_indicator_me_service_pretty_user_name (DBusGProxy *proxy, char ** OUT_name, GError **error)

{
  return dbus_g_proxy_call (proxy, "PrettyUserName", error, G_TYPE_INVALID, G_TYPE_STRING, OUT_name, G_TYPE_INVALID);
}

typedef void (*org_ayatana_indicator_me_service_pretty_user_name_reply) (DBusGProxy *proxy, char * OUT_name, GError *error, gpointer userdata);

static void
org_ayatana_indicator_me_service_pretty_user_name_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  char * OUT_name;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_STRING, &OUT_name, G_TYPE_INVALID);
  (*(org_ayatana_indicator_me_service_pretty_user_name_reply)data->cb) (proxy, OUT_name, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
org_ayatana_indicator_me_service_pretty_user_name_async (DBusGProxy *proxy, org_ayatana_indicator_me_service_pretty_user_name_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "PrettyUserName", org_ayatana_indicator_me_service_pretty_user_name_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_INVALID);
}
#endif /* defined DBUS_GLIB_CLIENT_WRAPPERS_org_ayatana_indicator_me_service */

G_END_DECLS
