
#ifndef __DBUS_DATA_H__
#define __DBUS_DATA_H__ 1

#define INDICATOR_MESSAGES_DBUS_NAME               "org.ayatana.indicator.messages"
#define INDICATOR_MESSAGES_DBUS_OBJECT             "/org/ayatana/indicator/messages/menu"

#define INDICATOR_MESSAGES_DBUS_SERVICE_OBJECT     "/org/ayatana/indicator/messages/service"
#define INDICATOR_MESSAGES_DBUS_SERVICE_INTERFACE  "org.ayatana.indicator.messages.service"

#define APPLICATION_MENUITEM_TYPE        "application-item"
#define APPLICATION_MENUITEM_PROP_NAME   "label"
#define APPLICATION_MENUITEM_PROP_ICON   "icon-name"
#define APPLICATION_MENUITEM_PROP_RUNNING "app-running"

#define INDICATOR_MENUITEM_TYPE          "indicator-item"
#define INDICATOR_MENUITEM_PROP_LABEL    "indicator-label"
#define INDICATOR_MENUITEM_PROP_ICON     "indicator-icon"
#define INDICATOR_MENUITEM_PROP_RIGHT    "right-side-text"
#if (INDICATOR_MESSAGES_HAS_LOZENGE == 1)
#define INDICATOR_MENUITEM_PROP_RIGHT_IS_LOZENGE "right-is-lozenge"
#endif

#define MAX_NUMBER_OF_INDICATORS  7

#endif /* __DBUS_DATA_H__ */
