# This is a part of the external demo applet for Cairo-Dock
# Copyright : (C) 2011 by matttbe
# E-mail : matttbe@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL

from setuptools import setup
setup(name='CDBashApplet',
		version='1.0',
		license='GPL-3',
		author='Fabrice Rey',
		author_email='fabounet@glx-dock.org',
		description='Bash interface for Cairo-Dock',
		url='https://launchpad.net/cairo-dock',
		py_modules=['CDBashApplet']
	)
