/*
** Login : <chris.chapuis@gmail.com>
** Started on  Thu Apr 03 18:21:35 2008 CHAPUIS Christophe
** $Id$
**
** Copyright (C) 2008 CHAPUIS Christophe
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __CD_WEBLETS_WIDGET__
#define  __CD_WEBLETS_WIDGET__

#include <cairo-dock.h>

CairoDialog *cd_weblets_build_dialog (GldiModuleInstance *myApplet);

gboolean cd_weblets_refresh_page (GldiModuleInstance *myApplet);

void weblet_build_and_show (GldiModuleInstance *myApplet);

#endif
